From 69c4f4eb8e0ed40c434867fbb740a68383623da9 Mon Sep 17 00:00:00 2001
From: Ryo Kanbayashi <ryo.contact@gmail.com>
Date: Sun, 23 Mar 2025 11:41:06 +0900
Subject: [PATCH v3 1/2] add regression test of service connection option.

---
 src/interfaces/libpq/meson.build      |  1 +
 src/interfaces/libpq/t/006_service.pl | 79 +++++++++++++++++++++++++++
 2 files changed, 80 insertions(+)
 create mode 100644 src/interfaces/libpq/t/006_service.pl

diff --git a/src/interfaces/libpq/meson.build b/src/interfaces/libpq/meson.build
index 19f4a52a97a..292fecf3320 100644
--- a/src/interfaces/libpq/meson.build
+++ b/src/interfaces/libpq/meson.build
@@ -122,6 +122,7 @@ tests += {
       't/003_load_balance_host_list.pl',
       't/004_load_balance_dns.pl',
       't/005_negotiate_encryption.pl',
+      't/006_service.pl',
     ],
     'env': {
       'with_ssl': ssl_library,
diff --git a/src/interfaces/libpq/t/006_service.pl b/src/interfaces/libpq/t/006_service.pl
new file mode 100644
index 00000000000..045e25a05d3
--- /dev/null
+++ b/src/interfaces/libpq/t/006_service.pl
@@ -0,0 +1,79 @@
+# Copyright (c) 2025, PostgreSQL Global Development Group
+use strict;
+use warnings FATAL => 'all';
+use Config;
+use PostgreSQL::Test::Utils;
+use PostgreSQL::Test::Cluster;
+use Test::More;
+
+# This tests "service" connection options.
+
+# Cluster setup which is shared for testing both load balancing methods
+my $node = PostgreSQL::Test::Cluster->new('node');
+
+# Create a data directory with initdb
+$node->init();
+
+# Start the PostgreSQL server
+$node->start();
+
+my $td      = PostgreSQL::Test::Utils::tempdir;
+my $srvfile = "$td/pgsrv.conf";
+
+# Windows: use CRLF
+# Non-Windows: use LF
+my $newline = $windows_os ? "\r\n" : "\n";
+
+# Create a service file
+open my $fh, '>', $srvfile or die $!;
+print $fh "[my_srv]",                                     $newline;
+print $fh join( $newline, split( ' ', $node->connstr ) ), $newline;
+
+close $fh;
+
+# Check that service option works as expected
+{
+    local $ENV{PGSERVICEFILE} = $srvfile;
+    $node->connect_ok(
+        'service=my_srv',
+        'service=my_srv',
+        sql             => "SELECT 'connect1'",
+        expected_stdout => qr/connect1/
+    );
+
+    $node->connect_ok(
+        'postgres://?service=my_srv',
+        'postgres://?service=my_srv',
+        sql             => "SELECT 'connect2'",
+        expected_stdout => qr/connect2/
+    );
+
+    local $ENV{PGSERVICE} = 'my_srv';
+    $node->connect_ok(
+        '',
+        'envvar: PGSERVICE=my_srv',
+        sql             => "SELECT 'connect3'",
+        expected_stdout => qr/connect3/
+    );
+}
+
+# Check that not existing service fails
+{
+    local $ENV{PGSERVICEFILE} = $srvfile;
+    local $ENV{PGSERVICE} = 'non-existent-service';
+    $node->connect_fails(
+        '',
+        'envvar: PGSERVICE=non-existent-service',
+        expected_stdout =>
+          qr/definition of service "non-existent-service" not found/
+    );
+
+    $node->connect_fails(
+        'service=non-existent-service',
+        'service=non-existent-service',
+        expected_stderr =>
+          qr/definition of service "non-existent-service" not found/
+    );
+}
+
+done_testing();
\ No newline at end of file
-- 
2.25.1

